################################################################################
#
# Copyright (c) 2002-2024 Marcus Holland-Moritz. All rights reserved.
# This program is free software; you can redistribute it and/or modify
# it under the same terms as Perl itself.
#
################################################################################

%size = (
	'FILE'	=>	 117,
	'FILE.bufend'	=>	   8,
	'FILE.buffer'	=>	   8,
	'FILE.bufidx'	=>	   8,
	'FILE.bufsize'	=>	   8,
	'FILE.filename'	=>	   8,
	'FILE.handle'	=>	   4,
	'FILE.mtx'	=>	  40,
	'FILE.mtx._PDCLIB_mtx_t_align'	=>	   8,
	'FILE.mtx._PDCLIB_mtx_t_data'	=>	  40,
	'FILE.mtx._PDCLIB_mtx_t_data[0]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[10]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[11]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[12]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[13]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[14]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[15]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[16]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[17]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[18]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[19]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[1]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[20]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[21]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[22]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[23]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[24]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[25]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[26]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[27]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[28]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[29]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[2]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[30]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[31]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[32]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[33]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[34]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[35]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[36]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[37]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[38]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[39]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[3]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[4]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[5]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[6]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[7]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[8]'	=>	   1,
	'FILE.mtx._PDCLIB_mtx_t_data[9]'	=>	   1,
	'FILE.next'	=>	   8,
	'FILE.pos'	=>	  12,
	'FILE.pos.offset'	=>	   8,
	'FILE.pos.status'	=>	   4,
	'FILE.status'	=>	   4,
	'FILE.ungetbuf'	=>	   1,
	'FILE.ungetbuf[0]'	=>	   1,
	'FILE.ungetidx'	=>	   8,
	'_PDCLIB_bigint_t'	=>	 132,
	'_PDCLIB_bigint_t.data'	=>	 128,
	'_PDCLIB_bigint_t.data[0]'	=>	   4,
	'_PDCLIB_bigint_t.data[10]'	=>	   4,
	'_PDCLIB_bigint_t.data[11]'	=>	   4,
	'_PDCLIB_bigint_t.data[12]'	=>	   4,
	'_PDCLIB_bigint_t.data[13]'	=>	   4,
	'_PDCLIB_bigint_t.data[14]'	=>	   4,
	'_PDCLIB_bigint_t.data[15]'	=>	   4,
	'_PDCLIB_bigint_t.data[16]'	=>	   4,
	'_PDCLIB_bigint_t.data[17]'	=>	   4,
	'_PDCLIB_bigint_t.data[18]'	=>	   4,
	'_PDCLIB_bigint_t.data[19]'	=>	   4,
	'_PDCLIB_bigint_t.data[1]'	=>	   4,
	'_PDCLIB_bigint_t.data[20]'	=>	   4,
	'_PDCLIB_bigint_t.data[21]'	=>	   4,
	'_PDCLIB_bigint_t.data[22]'	=>	   4,
	'_PDCLIB_bigint_t.data[23]'	=>	   4,
	'_PDCLIB_bigint_t.data[24]'	=>	   4,
	'_PDCLIB_bigint_t.data[25]'	=>	   4,
	'_PDCLIB_bigint_t.data[26]'	=>	   4,
	'_PDCLIB_bigint_t.data[27]'	=>	   4,
	'_PDCLIB_bigint_t.data[28]'	=>	   4,
	'_PDCLIB_bigint_t.data[29]'	=>	   4,
	'_PDCLIB_bigint_t.data[2]'	=>	   4,
	'_PDCLIB_bigint_t.data[30]'	=>	   4,
	'_PDCLIB_bigint_t.data[31]'	=>	   4,
	'_PDCLIB_bigint_t.data[3]'	=>	   4,
	'_PDCLIB_bigint_t.data[4]'	=>	   4,
	'_PDCLIB_bigint_t.data[5]'	=>	   4,
	'_PDCLIB_bigint_t.data[6]'	=>	   4,
	'_PDCLIB_bigint_t.data[7]'	=>	   4,
	'_PDCLIB_bigint_t.data[8]'	=>	   4,
	'_PDCLIB_bigint_t.data[9]'	=>	   4,
	'_PDCLIB_bigint_t.size'	=>	   4,
	'_PDCLIB_cnd_attr_t'	=>	   4,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_align'	=>	   4,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_data'	=>	   4,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_data[0]'	=>	   1,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_data[1]'	=>	   1,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_data[2]'	=>	   1,
	'_PDCLIB_cnd_attr_t._PDCLIB_cnd_attr_t_data[3]'	=>	   1,
	'_PDCLIB_cnd_t'	=>	  48,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_align'	=>	   8,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data'	=>	  48,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[0]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[10]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[11]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[12]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[13]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[14]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[15]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[16]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[17]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[18]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[19]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[1]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[20]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[21]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[22]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[23]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[24]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[25]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[26]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[27]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[28]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[29]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[2]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[30]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[31]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[32]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[33]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[34]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[35]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[36]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[37]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[38]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[39]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[3]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[40]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[41]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[42]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[43]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[44]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[45]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[46]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[47]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[4]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[5]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[6]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[7]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[8]'	=>	   1,
	'_PDCLIB_cnd_t._PDCLIB_cnd_t_data[9]'	=>	   1,
	'_PDCLIB_div_t'	=>	   8,
	'_PDCLIB_div_t.quot'	=>	   4,
	'_PDCLIB_div_t.rem'	=>	   4,
	'_PDCLIB_exitfunc_t'	=>	  16,
	'_PDCLIB_exitfunc_t.func'	=>	   8,
	'_PDCLIB_exitfunc_t.next'	=>	   8,
	'_PDCLIB_fd_t'	=>	   4,
	'_PDCLIB_file_t'	=>	 117,
	'_PDCLIB_file_t.bufend'	=>	   8,
	'_PDCLIB_file_t.buffer'	=>	   8,
	'_PDCLIB_file_t.bufidx'	=>	   8,
	'_PDCLIB_file_t.bufsize'	=>	   8,
	'_PDCLIB_file_t.filename'	=>	   8,
	'_PDCLIB_file_t.handle'	=>	   4,
	'_PDCLIB_file_t.mtx'	=>	  40,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_align'	=>	   8,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data'	=>	  40,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[0]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[10]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[11]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[12]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[13]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[14]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[15]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[16]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[17]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[18]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[19]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[1]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[20]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[21]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[22]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[23]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[24]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[25]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[26]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[27]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[28]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[29]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[2]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[30]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[31]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[32]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[33]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[34]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[35]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[36]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[37]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[38]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[39]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[3]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[4]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[5]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[6]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[7]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[8]'	=>	   1,
	'_PDCLIB_file_t.mtx._PDCLIB_mtx_t_data[9]'	=>	   1,
	'_PDCLIB_file_t.next'	=>	   8,
	'_PDCLIB_file_t.pos'	=>	  12,
	'_PDCLIB_file_t.pos.offset'	=>	   8,
	'_PDCLIB_file_t.pos.status'	=>	   4,
	'_PDCLIB_file_t.status'	=>	   4,
	'_PDCLIB_file_t.ungetbuf'	=>	   1,
	'_PDCLIB_file_t.ungetbuf[0]'	=>	   1,
	'_PDCLIB_file_t.ungetidx'	=>	   8,
	'_PDCLIB_fpos_t'	=>	  12,
	'_PDCLIB_fpos_t.offset'	=>	   8,
	'_PDCLIB_fpos_t.status'	=>	   4,
	'_PDCLIB_imaxdiv_t'	=>	  16,
	'_PDCLIB_imaxdiv_t.quot'	=>	   8,
	'_PDCLIB_imaxdiv_t.rem'	=>	   8,
	'_PDCLIB_lc_collate_t'	=>	   4,
	'_PDCLIB_lc_collate_t.alloced'	=>	   4,
	'_PDCLIB_lc_ctype_entry_t'	=>	   4,
	'_PDCLIB_lc_ctype_entry_t.flags'	=>	   2,
	'_PDCLIB_lc_ctype_entry_t.lower'	=>	   1,
	'_PDCLIB_lc_ctype_entry_t.upper'	=>	   1,
	'_PDCLIB_lc_ctype_t'	=>	  36,
	'_PDCLIB_lc_ctype_t.Xdigits_high'	=>	   4,
	'_PDCLIB_lc_ctype_t.Xdigits_low'	=>	   4,
	'_PDCLIB_lc_ctype_t.alloced'	=>	   4,
	'_PDCLIB_lc_ctype_t.digits_high'	=>	   4,
	'_PDCLIB_lc_ctype_t.digits_low'	=>	   4,
	'_PDCLIB_lc_ctype_t.entry'	=>	   8,
	'_PDCLIB_lc_ctype_t.xdigits_high'	=>	   4,
	'_PDCLIB_lc_ctype_t.xdigits_low'	=>	   4,
	'_PDCLIB_lc_lconv_monetary_t'	=>	  70,
	'_PDCLIB_lc_lconv_monetary_t.currency_symbol'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.frac_digits'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_curr_symbol'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.int_frac_digits'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_n_cs_precedes'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_n_sep_by_space'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_n_sign_posn'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_p_cs_precedes'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_p_sep_by_space'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.int_p_sign_posn'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.mon_decimal_point'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.mon_grouping'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.mon_thousands_sep'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.n_cs_precedes'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.n_sep_by_space'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.n_sign_posn'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.negative_sign'	=>	   8,
	'_PDCLIB_lc_lconv_monetary_t.p_cs_precedes'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.p_sep_by_space'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.p_sign_posn'	=>	   1,
	'_PDCLIB_lc_lconv_monetary_t.positive_sign'	=>	   8,
	'_PDCLIB_lc_lconv_numeric_t'	=>	  24,
	'_PDCLIB_lc_lconv_numeric_t.decimal_point'	=>	   8,
	'_PDCLIB_lc_lconv_numeric_t.grouping'	=>	   8,
	'_PDCLIB_lc_lconv_numeric_t.thousands_sep'	=>	   8,
	'_PDCLIB_lc_messages_t'	=>	1060,
	'_PDCLIB_lc_messages_t.alloced'	=>	   4,
	'_PDCLIB_lc_messages_t.errno_texts'	=>	1056,
	'_PDCLIB_lc_messages_t.errno_texts[0]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[100]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[101]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[102]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[103]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[104]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[105]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[106]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[107]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[108]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[109]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[10]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[110]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[111]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[112]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[113]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[114]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[115]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[116]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[117]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[118]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[119]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[11]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[120]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[121]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[122]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[123]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[124]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[125]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[126]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[127]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[128]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[129]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[12]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[130]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[131]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[13]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[14]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[15]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[16]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[17]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[18]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[19]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[1]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[20]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[21]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[22]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[23]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[24]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[25]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[26]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[27]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[28]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[29]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[2]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[30]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[31]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[32]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[33]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[34]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[35]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[36]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[37]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[38]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[39]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[3]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[40]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[41]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[42]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[43]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[44]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[45]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[46]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[47]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[48]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[49]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[4]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[50]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[51]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[52]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[53]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[54]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[55]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[56]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[57]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[58]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[59]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[5]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[60]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[61]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[62]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[63]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[64]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[65]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[66]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[67]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[68]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[69]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[6]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[70]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[71]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[72]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[73]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[74]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[75]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[76]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[77]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[78]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[79]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[7]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[80]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[81]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[82]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[83]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[84]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[85]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[86]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[87]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[88]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[89]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[8]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[90]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[91]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[92]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[93]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[94]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[95]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[96]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[97]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[98]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[99]'	=>	   8,
	'_PDCLIB_lc_messages_t.errno_texts[9]'	=>	   8,
	'_PDCLIB_lc_numeric_monetary_t'	=>	  16,
	'_PDCLIB_lc_numeric_monetary_t.lconv'	=>	   8,
	'_PDCLIB_lc_numeric_monetary_t.monetary_alloced'	=>	   4,
	'_PDCLIB_lc_numeric_monetary_t.numeric_alloced'	=>	   4,
	'_PDCLIB_lc_time_t'	=>	 356,
	'_PDCLIB_lc_time_t.alloced'	=>	   4,
	'_PDCLIB_lc_time_t.am_pm'	=>	  16,
	'_PDCLIB_lc_time_t.am_pm[0]'	=>	   8,
	'_PDCLIB_lc_time_t.am_pm[1]'	=>	   8,
	'_PDCLIB_lc_time_t.date_format'	=>	   8,
	'_PDCLIB_lc_time_t.date_time_format'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr'	=>	  56,
	'_PDCLIB_lc_time_t.day_name_abbr[0]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[1]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[2]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[3]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[4]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[5]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_abbr[6]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full'	=>	  56,
	'_PDCLIB_lc_time_t.day_name_full[0]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[1]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[2]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[3]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[4]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[5]'	=>	   8,
	'_PDCLIB_lc_time_t.day_name_full[6]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr'	=>	  96,
	'_PDCLIB_lc_time_t.month_name_abbr[0]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[10]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[11]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[1]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[2]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[3]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[4]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[5]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[6]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[7]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[8]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_abbr[9]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full'	=>	  96,
	'_PDCLIB_lc_time_t.month_name_full[0]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[10]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[11]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[1]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[2]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[3]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[4]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[5]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[6]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[7]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[8]'	=>	   8,
	'_PDCLIB_lc_time_t.month_name_full[9]'	=>	   8,
	'_PDCLIB_lc_time_t.time_format'	=>	   8,
	'_PDCLIB_lc_time_t.time_format_12h'	=>	   8,
	'_PDCLIB_ldiv_t'	=>	  16,
	'_PDCLIB_ldiv_t.quot'	=>	   8,
	'_PDCLIB_ldiv_t.rem'	=>	   8,
	'_PDCLIB_lldiv_t'	=>	  16,
	'_PDCLIB_lldiv_t.quot'	=>	   8,
	'_PDCLIB_lldiv_t.rem'	=>	   8,
	'_PDCLIB_mtx_attr_t'	=>	   4,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_align'	=>	   4,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_data'	=>	   4,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_data[0]'	=>	   1,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_data[1]'	=>	   1,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_data[2]'	=>	   1,
	'_PDCLIB_mtx_attr_t._PDCLIB_mtx_attr_t_data[3]'	=>	   1,
	'_PDCLIB_mtx_t'	=>	  40,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_align'	=>	   8,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data'	=>	  40,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[0]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[10]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[11]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[12]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[13]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[14]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[15]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[16]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[17]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[18]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[19]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[1]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[20]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[21]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[22]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[23]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[24]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[25]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[26]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[27]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[28]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[29]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[2]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[30]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[31]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[32]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[33]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[34]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[35]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[36]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[37]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[38]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[39]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[3]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[4]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[5]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[6]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[7]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[8]'	=>	   1,
	'_PDCLIB_mtx_t._PDCLIB_mtx_t_data[9]'	=>	   1,
	'_PDCLIB_once_flag'	=>	   4,
	'_PDCLIB_status_t'	=>	  72,
	'_PDCLIB_status_t.arg'	=>	   8,
	'_PDCLIB_status_t.base'	=>	   4,
	'_PDCLIB_status_t.current'	=>	   8,
	'_PDCLIB_status_t.flags'	=>	   8,
	'_PDCLIB_status_t.i'	=>	   8,
	'_PDCLIB_status_t.n'	=>	   8,
	'_PDCLIB_status_t.prec'	=>	   4,
	'_PDCLIB_status_t.s'	=>	   8,
	'_PDCLIB_status_t.stream'	=>	   8,
	'_PDCLIB_status_t.width'	=>	   8,
	'_PDCLIB_thrd_attr_t'	=>	  56,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_align'	=>	   8,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data'	=>	  56,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[0]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[10]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[11]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[12]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[13]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[14]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[15]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[16]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[17]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[18]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[19]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[1]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[20]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[21]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[22]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[23]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[24]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[25]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[26]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[27]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[28]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[29]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[2]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[30]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[31]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[32]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[33]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[34]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[35]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[36]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[37]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[38]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[39]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[3]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[40]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[41]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[42]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[43]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[44]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[45]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[46]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[47]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[48]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[49]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[4]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[50]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[51]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[52]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[53]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[54]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[55]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[5]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[6]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[7]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[8]'	=>	   1,
	'_PDCLIB_thrd_attr_t._PDCLIB_thrd_attr_t_data[9]'	=>	   1,
	'_PDCLIB_thrd_t'	=>	   8,
	'_PDCLIB_tss_t'	=>	   4,
	'_PDCLIB_va_list'	=>	   8,
	'bar'	=>	   8,
	'bar.x'	=>	   4,
	'bar.y'	=>	   8,
	'clock_t'	=>	   8,
	'div_t'	=>	   8,
	'div_t.quot'	=>	   4,
	'div_t.rem'	=>	   4,
	'foo'	=>	   1,
	'fpos_t'	=>	  12,
	'fpos_t.offset'	=>	   8,
	'fpos_t.status'	=>	   4,
	'imaxdiv_t'	=>	  16,
	'imaxdiv_t.quot'	=>	   8,
	'imaxdiv_t.rem'	=>	   8,
	'int16_t'	=>	   2,
	'int32_t'	=>	   4,
	'int64_t'	=>	   8,
	'int8_t'	=>	   1,
	'int_fast16_t'	=>	   8,
	'int_fast32_t'	=>	   8,
	'int_fast64_t'	=>	   8,
	'int_fast8_t'	=>	   1,
	'int_least16_t'	=>	   2,
	'int_least32_t'	=>	   4,
	'int_least64_t'	=>	   8,
	'int_least8_t'	=>	   1,
	'intmax_t'	=>	   8,
	'intptr_t'	=>	   8,
	'lconv'	=>	  94,
	'lconv.currency_symbol'	=>	   8,
	'lconv.decimal_point'	=>	   8,
	'lconv.frac_digits'	=>	   1,
	'lconv.grouping'	=>	   8,
	'lconv.int_curr_symbol'	=>	   8,
	'lconv.int_frac_digits'	=>	   1,
	'lconv.int_n_cs_precedes'	=>	   1,
	'lconv.int_n_sep_by_space'	=>	   1,
	'lconv.int_n_sign_posn'	=>	   1,
	'lconv.int_p_cs_precedes'	=>	   1,
	'lconv.int_p_sep_by_space'	=>	   1,
	'lconv.int_p_sign_posn'	=>	   1,
	'lconv.mon_decimal_point'	=>	   8,
	'lconv.mon_grouping'	=>	   8,
	'lconv.mon_thousands_sep'	=>	   8,
	'lconv.n_cs_precedes'	=>	   1,
	'lconv.n_sep_by_space'	=>	   1,
	'lconv.n_sign_posn'	=>	   1,
	'lconv.negative_sign'	=>	   8,
	'lconv.p_cs_precedes'	=>	   1,
	'lconv.p_sep_by_space'	=>	   1,
	'lconv.p_sign_posn'	=>	   1,
	'lconv.positive_sign'	=>	   8,
	'lconv.thousands_sep'	=>	   8,
	'ldiv_t'	=>	  16,
	'ldiv_t.quot'	=>	   8,
	'ldiv_t.rem'	=>	   8,
	'lldiv_t'	=>	  16,
	'lldiv_t.quot'	=>	   8,
	'lldiv_t.rem'	=>	   8,
	'ptrdiff_t'	=>	   8,
	'size_t'	=>	   8,
	'time_t'	=>	   8,
	'timespec'	=>	  16,
	'timespec.tv_nsec'	=>	   8,
	'timespec.tv_sec'	=>	   8,
	'tm'	=>	  36,
	'tm.tm_hour'	=>	   4,
	'tm.tm_isdst'	=>	   4,
	'tm.tm_mday'	=>	   4,
	'tm.tm_min'	=>	   4,
	'tm.tm_mon'	=>	   4,
	'tm.tm_sec'	=>	   4,
	'tm.tm_wday'	=>	   4,
	'tm.tm_yday'	=>	   4,
	'tm.tm_year'	=>	   4,
	'uint16_t'	=>	   2,
	'uint32_t'	=>	   4,
	'uint64_t'	=>	   8,
	'uint8_t'	=>	   1,
	'uint_fast16_t'	=>	   8,
	'uint_fast32_t'	=>	   8,
	'uint_fast64_t'	=>	   8,
	'uint_fast8_t'	=>	   1,
	'uint_least16_t'	=>	   2,
	'uint_least32_t'	=>	   4,
	'uint_least64_t'	=>	   8,
	'uint_least8_t'	=>	   1,
	'uintmax_t'	=>	   8,
	'uintptr_t'	=>	   8,
	'va_list'	=>	   8,
	'wchar_t'	=>	   4,
	'wctrans_t'	=>	   4,
	'wctype_t'	=>	   4,
	'wint_t'	=>	   4,
);
